
/* PC1450 - A calculator emulator
 * Copyright (C) 1996 - 2001 Yves Maingoy
 * ymaingoy@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
    video.c
    version Windows 32 
    25/07/1997 et 23/11/2001
*/
#include "Pc1450.h"
#include "video.h"
#include "memory.h"

#define ON  (0==0)
#define OFF (0==1)

int CurrentY = 0;
int isScreenRefresh=1; /* != 0 si on doit rafraichir l'ecran */
int ScreenON = 0; /* !=0 si Screen ON */

// Definition des sprites ecrans :
RECT rcBus  = { 57, 32,  23,   7};
RECT rcDef  = { 90, 32,  15,   7};
RECT rcShi  = {142, 32,  24,   7};
RECT rcHyp  = {176, 32,  17,   7};
RECT rcSml  = {220, 32,  16,   7};
RECT rcDeg  = {239, 32,  11,   7};
RECT rcGra  = {251, 32,   5,   7};
RECT rcRad  = {257, 32,  17,   7};
RECT rcPar  = {312, 32,  10,   7};
RECT rcMem  = {330, 32,   7,   7};
RECT rcErr  = {339, 32,   4,   7};

RECT rcjap1 = {198, 32,   8,   7};
RECT rcjap2 = {211, 32,   5,   7};

RECT rcCal  = { 75, 80,   4,   2};
RECT rcRun  = {104, 80,   4,   2};
RECT rcPro  = {129, 80,   4,   2};
RECT rcSta  = {285, 80,   4,   2};
RECT rcPri  = {311, 80,   4,   2};

/****************************************************************************

    FUNCTION:  Sprite(HWND hWnd, RECT rc, int etat)

    PURPOSE:   Affiche un sprite selon l'etat ON ou OFF

    COMMENTS:  

****************************************************************************/
void Sprite(HWND hWnd, RECT rc, int etat)
{
    HBITMAP hbmpSave;      // variable de sauvegarde

    // prepare le contexte ecran
    hDC = GetDC(hWnd);


    // recupere le contexte de la memoire ecran
    //hdcMemory = CreateCompatibleDC(hdc);

    // Selection du Bmp a afficher, sauve le contexte
    hbmpSave = SelectObject(hMemoryDC, hEcranBitmap);

    // Affiche a l'ecran le bitmap
    if(!etat)
        BitBlt(hDC, rc.left, rc.top, rc.right, rc.bottom, hMemoryDC, rc.left, rc.top+rc.bottom, SRCCOPY);
    else
        BitBlt(hDC, rc.left, rc.top, rc.right, rc.bottom, hMemoryDC, rc.left, rc.top, SRCCOPY);


    // Restaure le contexte ecran
    SelectObject(hMemoryDC, hbmpSave);

    // libere la memoire
    // (fait en dernier dans l'application)
    // DeleteDC(hdcMemory);

    // libere le contexte
    ReleaseDC(hWnd, hDC);
}

/****************************************************************************

    FUNCTION:  RefreshSpecial(HWND hWnd)

    PURPOSE:   Affiche les sprites speciaux selon certaines memoires

    COMMENTS:  
               
****************************************************************************/
void RefreshSpecial(HWND hWnd)
{
    unsigned char oct;

    oct = peekPC((UINT) 0x703D);
    if (oct & 0x01) Sprite(hWnd, rcBus, ON);
    else            Sprite(hWnd, rcBus, OFF);
    if (oct & 0x02) Sprite(hWnd, rcDef, ON);
    else            Sprite(hWnd, rcDef, OFF);
    if (oct & 0x04) Sprite(hWnd, rcShi, ON);
    else            Sprite(hWnd, rcShi, OFF);
    if (oct & 0x08) Sprite(hWnd, rcHyp, ON);
    else            Sprite(hWnd, rcHyp, OFF);

    if (oct & 0x10) Sprite(hWnd, rcPro, ON);
    else            Sprite(hWnd, rcPro, OFF);
    if (oct & 0x20) Sprite(hWnd, rcRun, ON);
    else            Sprite(hWnd, rcRun, OFF);
    if (oct & 0x40) Sprite(hWnd, rcCal, ON);
    else            Sprite(hWnd, rcCal, OFF);
/*  if (oct & 0x80) Sprite(hWnd, rcSml, ON);
    else            Sprite(hWnd, rcSml, OFF);
*/
    oct = peekPC((UINT) 0x707C);
    if (oct & 0x01) Sprite(hWnd, rcErr, ON);
    else            Sprite(hWnd, rcErr, OFF);
    if (oct & 0x02) Sprite(hWnd, rcMem, ON);
    else            Sprite(hWnd, rcMem, OFF);
    if (oct & 0x04) Sprite(hWnd, rcPar, ON);
    else            Sprite(hWnd, rcPar, OFF);
    if (oct & 0x08) Sprite(hWnd, rcRad, ON);
    else            Sprite(hWnd, rcRad, OFF);

    if (oct & 0x10) Sprite(hWnd, rcGra, ON);
    else            Sprite(hWnd, rcGra, OFF);
    if (oct & 0x20) Sprite(hWnd, rcDeg, ON);
    else            Sprite(hWnd, rcDeg, OFF);
    if (oct & 0x40) Sprite(hWnd, rcPri, ON);
    else            Sprite(hWnd, rcPri, OFF);

    oct = peekPC((UINT) 0x703C);
    if (oct & 0x01) Sprite(hWnd, rcjap1, ON);
    else            Sprite(hWnd, rcjap1, OFF);
    if (oct & 0x02) Sprite(hWnd, rcjap2, ON);
    else            Sprite(hWnd, rcjap2, OFF);
    if (oct & 0x04) Sprite(hWnd, rcSml, ON);
    else            Sprite(hWnd, rcSml, OFF);
    if (oct & 0x08) Sprite(hWnd, rcSta, ON);
    else            Sprite(hWnd, rcSta, OFF);

/*  if (oct & 0x10) Sprite(hWnd, rcSml, ON);
    else            Sprite(hWnd, rcSml, OFF);
    if (oct & 0x20) Sprite(hWnd, rcSml, ON);
    else            Sprite(hWnd, rcSml, OFF);
    if (oct & 0x40) Sprite(hWnd, rcSml, ON);
    else            Sprite(hWnd, rcSml, OFF);
    if (oct & 0x80) Sprite(hWnd, rcSml, ON);
    else            Sprite(hWnd, rcSml, OFF);
*/
}

/****************************************************************************

    FUNCTION:  PutByte(HWND, int, int, uchar)

    PURPOSE:   Affiche en (X,Y) une colonne de pixels selon le masque byte

    COMMENTS:  Utilise les variables globales suivantes
               hMemoryDC
               hPixelsBitmap

****************************************************************************/
void PutByte(HWND hWnd, int X, int Y, unsigned char byte)
{
    HBITMAP hbmpSave;      // variable de sauvegarde
    int Xmask;
    int const X0 = 65; // premier pixel de l'ecran P(65,40)
    int const Y0 = 50;

    int const HEIGHTPIXEL = 2; // 1 pixel calulette = 2*2 pixels
    int const HEIGHTCOLUNUM = 21; // (HEIGHTPIXEL +1) * 7-1

    // prepare le contexte ecran
    hDC = GetDC(hWnd);

    // recupere le contexte de la memoire ecran
    //hdcMemory = CreateCompatibleDC(hdc);

    // Selection du Bmp a afficher, sauve le contexte
    hbmpSave = SelectObject(hMemoryDC, hPixelsBitmap);

    // Affiche a l'ecran le bitmap
    // selon la valeur de l'octet passe en paremetre
    Xmask = (byte & 127)*HEIGHTPIXEL;
    BitBlt(hDC, X * (HEIGHTPIXEL+1) + X0, Y + Y0, HEIGHTPIXEL, HEIGHTCOLUNUM, hMemoryDC, Xmask, 0, SRCCOPY);

    // Restaure le contexte ecran
    SelectObject(hMemoryDC, hbmpSave);

    // libere la memoire
    // (fait en dernier dans l'application)
    // DeleteDC(hdcMemory);

    // libere le contexte
    ReleaseDC(hWnd, hDC);
}


/****************************************************************************

    FUNCTION:  PrintScreen(HWND hWnd, int Y,unsigned char * buffer)

    PURPOSE:   Affichage de l'ecran du Sharp

    COMMENTS:  
               12 lettres mode normal
               4  lettres mode inverse

****************************************************************************/
void PrintScreen(HWND hWnd, int Y,unsigned char * buffer)
{
    int i,j;
    int X;

    //if(isScreenRefresh != 0) {


        X=0;

        /* premiere memoire video du Sharp */
        for(i=0; i<12; i++) {
            for(j=0; j<5; j++)
                PutByte(hWnd, X++, Y, buffer[j]);
            X++;
            buffer+=5;
        }

        /* seconde memoire video du Sharp */
        buffer+=0x3A;
        for(i=0; i<4; i++) {
            for(j=5; j!=0; j--)
                PutByte(hWnd, X++, Y, buffer[j]);
            X++;
            buffer-=5;
        }

        /* les caracteres d'etats de l'ecran */
        RefreshSpecial(hWnd);

        /* L'ecran est rafraichi pour l'instant */
        //isScreenRefresh = 0;
    //}

}

/****************************************************************************

    FUNCTION:  ClrScreen(HWND hWnd, int Y)

    PURPOSE:   Efface les pixels de l'ecran du Sharp 

    COMMENTS:  
               16 lettres

****************************************************************************/
void ClrScreen(HWND hWnd, int Y)
{
    int i,j;
    int X;

    X=0;
    for(i=0; i<16; i++) {
        for(j=0; j<5; j++)
            PutByte(hWnd, X++, Y, 0);
        X++;
    }
}


/****************************************************************************

    FUNCTION:  AfficheEcran(HWND hWnd)

    PURPOSE:   affiche l'ecran du Sharp 

    COMMENTS:  


****************************************************************************/
void AfficheEcran(HWND hWnd)
{
    HBITMAP hbmpSave;      // variable de sauvegarde
        // prepare le contexte ecran
    hDC = GetDC(hWnd);

    // recupere le contexte de la memoire ecran
    //hdcMemory = CreateCompatibleDC(hdc);

    // Selection du Bmp a afficher, sauve le contexte
    hbmpSave = SelectObject(hMemoryDC, hEcranBitmap);

    // Affiche a l'ecran le bitmap
    BitBlt(hDC, 0, 0, WHIDTHBMP, HEIGHTBMP, hMemoryDC, 0, 0, SRCCOPY);

    // Restaure le contexte ecran
    SelectObject(hMemoryDC, hbmpSave);

    // libere la memoire
    // (fait en dernier dans l'application)
    // DeleteDC(hdcMemory);

    // libere le contexte
    ReleaseDC(hWnd, hDC);
}

/****************************************************************************

    FUNCTION:  PutLargePixel(HWND hWnd, POINT ptCurrent)

    PURPOSE:   Affiche un gros pixel sur l'ecran du sharp 

    COMMENTS:  

****************************************************************************/
void PutLargePixel(HWND hWnd, POINT ptCurrent)
{
    // Charge PixelBlanc avec le Bitmap2
    hPixelsBitmap = SelectObject(hMemoryDC, hBitmap2);
    GetObject(hBitmap2, 16, (LPSTR) &PixelBlanc);
    hDC = GetDC(hWnd);
    BitBlt(hDC,ptCurrent.x, ptCurrent.y,  3,3,
            hMemoryDC, 0, 0,  SRCCOPY);

    ReleaseDC(hWnd, hDC);
    hOldBitmap = SelectObject(hMemoryDC, hBitmap1);
}

/****************************************************************************

    FUNCTION:  ToucheUp(HWND hWnd, RECT rc)

    PURPOSE:   relache une touche enfoncee

    COMMENTS:  

****************************************************************************/
void ToucheUp(HWND hWnd, RECT rc)
{
    HDC hDC;
    HPEN hpenGray, hpenWhite, hpenOld;

    // prepare le contexte ecran
    hDC = GetDC(hWnd);

    hpenGray = CreatePen(PS_SOLID, 1, RGB(128, 128, 128)); //Gris fonce
    hpenWhite = CreatePen(PS_SOLID, 1, RGB(255, 255, 255)); //Blanc
    hpenOld = SelectObject(hDC, hpenWhite);

    MoveToEx(hDC, rc.left           , rc.top + rc.bottom -1, NULL);
    LineTo(hDC, rc.left           , rc.top);
    LineTo(hDC, rc.left + rc.right, rc.top);

    SelectObject(hDC, hpenGray);
    MoveToEx(hDC, rc.left + rc.right, rc.top + 1, NULL);
    LineTo(hDC, rc.left + rc.right, rc.top + rc.bottom);
    LineTo(hDC, rc.left           , rc.top + rc.bottom);

    // Restitue l'ancien contexte
    SelectObject(hDC, hpenOld);

    DeleteObject(hpenWhite);
    DeleteObject(hpenGray);
    ReleaseDC(hWnd, hDC);

}

/****************************************************************************

    FUNCTION:  ToucheDown(HWND hWnd, POINTS Cursor, RECT* rc)

    PURPOSE:   simule a l'ecran l'appuis sur une touche

    COMMENTS:  
              Cette fonction est complexe et possede de nombreux bugs.
****************************************************************************/
int ToucheDown(HWND hWnd, POINTS Cursor, RECT* rc)
{
    HDC hDC;
    POINT pt,pttmp;
    HPEN hpenGray1,hpenGray2, hpenOld;
    int ID_Touche=0; // code de la touche retournee

    //pt A(x,y) =(8,104)
    const int XKEYA=8;
    const int YKEYA=114;
    const int WHIDTHZONEA = 34;
    const int HEIGHTZONEA = 28;
    const int WHIDTHKEYA = 26;
    const int HEIGHTKEYA = 15;

    //pt B(x,y) =(386,85)
    const int XKEYB=386;
    const int YKEYB=95;
    const int WHIDTHZONEB = 42;
    const int HEIGHTZONEB = 32;
    const int WHIDTHKEYB = 30;
    const int HEIGHTKEYB = 22;

    //pt C(x,y) =(386,4)
    const int XKEYC=386;
    const int YKEYC=14;
    const int WHIDTHZONEC = 34;
    const int HEIGHTZONEC = 28;
    const int WHIDTHKEYC = 26;
    const int HEIGHTKEYC = 15;

    //pt D(x,y) =(314,188)
    const int XKEYD=314;
    const int YKEYD=198;
    const int WHIDTHZONED = 72;
    const int HEIGHTZONED = 24;
    const int WHIDTHKEYD = 60;
    const int HEIGHTKEYD = 15;


    // zone droite (zones C et B)
    if( (Cursor.x>XKEYB) )
    {
        // Zone droite haute (zone C)
        if( (Cursor.y>YKEYC) && (Cursor.y<YKEYB) )
        {
            pt.x =  Cursor.x - XKEYC;
            pt.y =  Cursor.y - YKEYC;
            pttmp.x =  pt.x/WHIDTHZONEC;
            pttmp.y =  pt.y/HEIGHTZONEC;
            ID_Touche = 45 + pttmp.x + 6*pttmp.y;
            pttmp.x =  WHIDTHZONEC * pttmp.x;
            pttmp.y =  HEIGHTZONEC * pttmp.y;
            if( (pt.x - pttmp.x<WHIDTHKEYC) && (pt.y - pttmp.y<HEIGHTKEYC) )
            {
                // remplir les coordonnees du bouton
                rc->left = XKEYC+1 + pttmp.x;
                rc->top  = YKEYC+1 + pttmp.y;
                rc->right= WHIDTHKEYC-3;
                rc->bottom=HEIGHTKEYC-3;
            }
            else ID_Touche = 0;
        }
        else
        {
            // Zone droite basse (zone B)
            if( (Cursor.y>YKEYB) )
            // test sur la zone droite basse
            {
                pt.x =  Cursor.x - XKEYB;
                pt.y =  Cursor.y - YKEYB;
                pttmp.x =  pt.x/WHIDTHZONEB;
                pttmp.y =  pt.y/HEIGHTZONEC;

                ID_Touche = 63 + pttmp.x + 5*pttmp.y;
                pttmp.x =  WHIDTHZONEB * pttmp.x;
                pttmp.y =  HEIGHTZONEB * pttmp.y;
                if( (pt.x - pttmp.x<WHIDTHKEYB) && (pt.y - pttmp.y<HEIGHTKEYB) )
                {
                    // remplir les coordonnees du bouton
                    rc->left = XKEYB+1 + pttmp.x; //x
                    rc->top  = YKEYB+1 + pttmp.y;    //y
                    rc->right= WHIDTHKEYB-3;
                    rc->bottom=HEIGHTKEYB-3;
                }
                else ID_Touche = 0;
            }
        }
    }
    else // zone gauche
    if( (Cursor.x>XKEYA) && (Cursor.x<XKEYB) && (Cursor.y>YKEYA) )
    {
        pt.x =  Cursor.x - XKEYA;
        pt.y =  Cursor.y - YKEYA;
        pttmp.x =  pt.x/WHIDTHZONEA;
        pttmp.y =  pt.y/HEIGHTZONEA;
   /*     if((pttmp.y==0) && (pttmp.x>2)&& (pttmp.x<5) )
        {
            //pas de touche dans cette zone
        }
        else
        { */
            ID_Touche = 1 + pttmp.x + 11*pttmp.y;
            pttmp.x =  WHIDTHZONEA * pttmp.x;
            pttmp.y =  HEIGHTZONEA * pttmp.y;

            // zone D (touche ENTER)
            if( (Cursor.x>XKEYD) && (Cursor.y>YKEYD) )
            {
                ID_Touche = 43; // code de ENTER

                //gestion de la touche ENTER
                rc->left   = XKEYD+1; //x
                rc->top    = YKEYD+1;  //y
                rc->right  = WHIDTHKEYD-3;
                rc->bottom = HEIGHTKEYD-3;
            }
            else
            if( (pt.x - pttmp.x<WHIDTHKEYA) && (pt.y - pttmp.y<HEIGHTKEYA) )
            {
                // remplir les coordonnees du bouton
                rc->left   = XKEYA+1 + pttmp.x; //x
                rc->top    = YKEYA+1+ pttmp.y;    //y
                rc->right  = WHIDTHKEYA-3;
                rc->bottom = HEIGHTKEYA-3;
            }
            else ID_Touche = 0;

   //     }

    }
        if(ID_Touche!=0)
        {

            // prepare le contexte ecran
            hDC = GetDC(hWnd);
            hpenGray1 = CreatePen(PS_SOLID, 1, RGB(128, 128, 128)); //Gris fonce
            hpenGray2 = CreatePen(PS_SOLID, 1, RGB(192, 192, 192)); //Gris clair

            hpenOld = SelectObject(hDC, hpenGray1);
            MoveToEx(hDC, rc->left            , rc->top + rc->bottom + 1, NULL);
            LineTo(hDC, rc->left            , rc->top);
            LineTo(hDC, rc->left + rc->right, rc->top);

            SelectObject(hDC, hpenGray2);
            MoveToEx(hDC, rc->left + rc->right, rc->top + 1, NULL);
            LineTo(hDC, rc->left + rc->right, rc->top + rc->bottom);
            LineTo(hDC, rc->left            , rc->top + rc->bottom);

            // Restitue l'ancien contexte
            SelectObject(hDC, hpenOld);

            DeleteObject(hpenGray2);
            DeleteObject(hpenGray1);
            ReleaseDC(hWnd, hDC);
        }

    // revoie le code de la touche slectionne
    return(ID_Touche);
}
